@@ -6,6 +6,7 @@
 */
 #include "survS.h"
 #include "R_ext/Rdynload.h"
+#include "Rversion.h"
 #include "survproto.h"
 
 static const R_CMethodDef Centries[] = {
@@ -45,6 +46,9 @@ static const R_CallMethodDef Callentries[] = {
     {"Ccoxexact",     (DL_FUNC) &coxexact,     8},
     {"Ccoxfit6",      (DL_FUNC) &coxfit6,     12},
     {"Cfinegray",     (DL_FUNC) &finegray,     6},
+    {"Cgchol",        (DL_FUNC) &gchol,        2},
+    {"Cgchol_solve",  (DL_FUNC) &gchol_solve,  3},
+    {"Cgchol_inv",    (DL_FUNC) &gchol_inv,    2},
     {"Cpyears3b",     (DL_FUNC) &pyears3b,    10},
     {"Csurvfitci",    (DL_FUNC) &survfitci,   11},
     {"Csurvreg6",     (DL_FUNC) &survreg6,    15},
@@ -65,7 +69,10 @@ void R_init_survival(DllInfo *dll){
     /*
     ** This line makes them only be available via the symbols above
     **  i.e., .Call("tmerge", ) won't work but .Call(Ctmerge, )  will
+    ** This feature was added in version 2.16
     */
+#if defined(R_VERSION) && R_VERSION >= R_Version(2, 16, 0)
     R_forceSymbols(dll, TRUE);
+#endif
 }
     
